﻿using System;
using System.Configuration;

namespace CYQ.Data
{
    public class AppConfig
    {
        private static string _CDataLeft = null;
        public static string CDataLeft
        {
            get
            {
                if (_CDataLeft == null)
                {
                    _CDataLeft = GetApp("CDataLeft");
                    if (_CDataLeft == null)
                    {
                        _CDataLeft = "<![CDATA[MMS::";
                    }
                }
                return _CDataLeft;
            }
        }
        private static string _CDataRight = null;
        public static string CDataRight
        {
            get
            {
                if (_CDataRight == null)
                {
                    _CDataRight = GetApp("CDataRight");
                    if (_CDataRight == null)
                    {
                        _CDataRight = "::MMS]]>";
                    }
                }
                return _CDataRight;
            }
        }

        public static string GetApp(string key)
        {
            return Convert.ToString(ConfigurationManager.AppSettings[key]);
        }
        public static int GetAppInt(string key, int defaultValue)
        {
            int result = 0;
            string value = GetApp(key);
            if (!int.TryParse(value, out result) || result == 0)
            {
                return defaultValue;
            }
            return result;
        }
        public static string GetConn(string key)
        {
            ConnectionStringSettings logConn = ConfigurationManager.ConnectionStrings[key];
            if (logConn != null)
            {
                return logConn.ConnectionString;
            }
            return "";
        }
        /// <summary>
        /// Aop程序集[程序集名称,类名]
        /// </summary>
        public static string Aop
        {
            get
            {
                return GetApp("Aop");
            }
        }

        public static bool IsWriteLog
        {
            get
            {
                bool _IsWriteLog;
                bool.TryParse(GetApp("IsWriteLog"), out _IsWriteLog);
                return _IsWriteLog;
            }
        }
        public static string LogConn
        {
            get
            {
                return GetConn("LogConn");
            }
        }

        /// <summary>
        /// Oracle 序列ID
        /// </summary>
        public static string AutoID
        {
            get
            {
                return GetApp("AutoID");
            }
        }
        public static string DtdUri
        {
            get
            {
                string dtdUri = GetApp("DtdUri");
                if (dtdUri != null && dtdUri.IndexOf("http://") == -1)//相对路径
                {
                    dtdUri = AppDomain.CurrentDomain.BaseDirectory + dtdUri.TrimStart('/');
                }
                return dtdUri;
            }
        }
        /// <summary>
        /// 多国语言时网站主域名[不带www]
        /// </summary>
        public static string Domain
        {
            get
            {
                return GetApp("Domain");
            }
        }
        #region 缓存相关配置


        /// <summary>
        /// 默认所有缓存项的时间[分钟(默认30)]
        /// </summary>
        public static int DefaultCacheTime
        {
            get
            {
                return GetAppInt("DefaultCacheTime", 30);
            }
        }
        /// <summary>
        /// 缓存同步时间[(默认5)分钟同步一次]
        /// </summary>
        public static int CacheClearWorkTime
        {
            get
            {
                return GetAppInt("CacheClearWorkTime", 5);
            }
        }
        /// <summary>
        /// N(默认4)分钟内调用次数少于指定值(默认2)，缓存即被清除[最少调用次数设置]
        /// </summary>
        public static int CacheClearCallCount
        {
            get
            {
                return GetAppInt("CacheClearCallCount", 2);
            }
        }
        /// <summary>
        /// N(默认4)分钟内调用次数少于指定值(默认2)，缓存即被清除[时间设置]
        /// </summary>
        public static int CacheClearTime
        {
            get
            {
                return GetAppInt("CacheClearTime", 4);
            }
        }

        #endregion
        ///// <summary>
        ///// Access数据库名称Web站
        ///// </summary>
        //public static string AccessDbNameForWeb
        //{
        //    get
        //    {
        //        return GetApp("AccessDbNameForWeb");
        //    }
        //}
        ///// <summary>
        ///// Access数据库名称Web站
        ///// </summary>
        //public static string AccessDbNameForApp
        //{
        //    get
        //    {
        //        return GetApp("AccessDbNameForApp");
        //    }
        //}
        ///// <summary>
        ///// SQLite数据库名称Web站
        ///// </summary>
        //public static string SQLiteDbNameForWeb
        //{
        //    get
        //    {
        //        return GetApp("SQLiteDbNameForWeb");
        //    }
        //}
        ///// <summary>
        ///// SQLite数据库名称Web站
        ///// </summary>
        //public static string SQLiteDbNameForApp
        //{
        //    get
        //    {
        //        return GetApp("SQLiteDbNameForApp");
        //    }
        //}
        /// <summary>
        /// 使用File加载Xml[在IIS7以上非信任主机机制下使用]
        /// </summary>
        public static bool UseFileLoadXml
        {
            get
            {
                bool _UseFileLoadXml;
                bool.TryParse(GetApp("UseFileLoadXml"), out _UseFileLoadXml);
                return _UseFileLoadXml;
            }
        }
        /// <summary>
        /// 写日志出现异常时，记录到文本文件路径
        /// </summary>
        public static string LogPath
        {
            get
            {
                return GetApp("LogPath");
            }
        }
        /// <summary>
        /// 系统默认语言Key
        /// </summary>
        public static string SysLangKey
        {
            get
            {
                return GetApp("SysLangKey");
            }
        }
        /// <summary>
        /// 开启信息调试记录：开启后MAction.DebugInfo可输出执行日志。
        /// 同时AppDebug若要使用，也需要开启此项。
        /// </summary>
        public static bool OpenDebugInfo
        {
            get
            {
                bool _OpenDebugInfo;
                bool.TryParse(GetApp("OpenDebugInfo"), out _OpenDebugInfo);
                return _OpenDebugInfo;
            }
        }
        /// <summary>
        /// CYQ.Data 数据库操作时,记录超长时间的SQL语句(默认0ms)，写入的文件路径为LogPath的路径
        /// </summary>
        public static int RecordSqlLongQueryTime
        {
            get
            {
                return GetAppInt("RecordSqlLongQueryTime", 0);
            }
        }
        /// <summary>
        /// CYQ.Data.AppDebug 使用时，打印一个页面的记录太多，通过此项可以过滤出时间较长的SQL语句。(默认0ms)
        /// </summary>
        public static int AppDebugFilterTime
        {
            get
            {
                return GetAppInt("AppDebugFilterTime", 0);
            }
        }
        /// <summary>
        /// Mssql/Oracle 默认是否开启事务[默认值为false]
        /// </summary>
        public static bool TransationDefaultOpen
        {
            get
            {
                bool _TransationDefaultIsOpen;
                bool.TryParse(GetApp("TransationDefaultIsOpen"), out _TransationDefaultIsOpen);
                return _TransationDefaultIsOpen;
            }
        }
        /// <summary>
        /// MAction所有操作中的where条件，默认有超强的过滤单词，来过滤Sql注入关键字。
        /// 如果需要自定义关键字，可配置此项，如：“delete;from,truncate，其它单词”，分号表词组，需要同时包含两个词； 多个过滤词组以","逗号分隔
        /// </summary>
        public static string FilterSqlInjection
        {
            get
            {
                return GetApp("FilterSqlInjection");
            }
        }
    }
}
