﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Caching;
namespace CYQ.Data.Cache
{
    /// <summary>
    /// 全局缓存类
    /// </summary>
    /// <example><code>
    /// 使用示例：
    /// 实例化： CacheManage cache=CacheManage.Instance;
    /// 添加：   cache.Add("路过秋天",new MDataTable);
    /// 判断：   if(cache.Contains("路过秋天"))
    ///          {
    /// 获取：       MDataTable table=cache.Get("路过秋天") as MDataTable;
    ///          }
    /// </code></example>
    public class CacheManage
    {
        private readonly System.Web.HttpContext H = new System.Web.HttpContext(new System.Web.HttpRequest("Null.File", "http://cyq1162.cnblogs.com", String.Empty), new System.Web.HttpResponse(null));
        private System.Web.Caching.Cache theCache = null;

        private CacheManage()
        {
            theCache = H.Cache;
        }
        /// <summary>
        /// 获和缓存总数
        /// </summary>
        public int Count
        {
            get
            {
                return theCache == null ? 0 : theCache.Count;
            }
        }
        /// <summary>
        /// 返回唯一实例
        /// </summary>
        public static CacheManage Instance
        {
            get
            {
                return Shell.instance;
            }
        }

        class Shell
        {
            internal static readonly CacheManage instance = new CacheManage();
        }

        /// <summary>
        /// 获得一个Cache对象
        /// </summary>
        /// <param name="key">标识</param>
        /// <returns></returns>
        public object Get(string key)
        {
            if (theCache != null && theCache[key] != null)
            {
                return theCache[key];
            }
            return null;
        }
        /// <summary>
        /// 是否存在缓存
        /// </summary>
        /// <param name="key">标识</param>
        /// <returns></returns>
        public bool Contains(string key)
        {
            return theCache != null && theCache[key] != null;
        }
        /// <summary>
        /// 添加一个Cache对象
        /// </summary>
        /// <param name="key">标识</param>
        /// <param name="value">对象值</param>
        public void Add(string key, object value)
        {
            if (!Contains(key))
            {
                Insert(key, value);//再插入Cache
            }
        }
        /// <summary>
        /// 相对底层Cache添加方法,添加一个Cache请用Add方法
        /// </summary>
        /// <param name="key">标识</param>
        /// <param name="value">对象值</param>
        private void Insert(string key, object value)
        {
            theCache.Insert(key, value, null,DateTime.Now.AddMinutes(20),TimeSpan.Zero, CacheItemPriority.Default,null);
        }

        /// <summary>
        /// 删除一个Cache对象
        /// </summary>
        /// <param name="key">标识</param>
        public void Remove(string key)
        {
            if (Contains(key))
            {
                theCache.Remove(key);
            }
        }
    }
}
