﻿using System;
using System.Collections.Generic;
using System.Text;
using CYQ.Data.Table;
using System.Reflection;

namespace CYQ.Data.Orm
{
    /// <summary>
    /// ORM扩展基类
    /// </summary>
    public abstract class OrmBase : ICommon
    {
        Object entity;//实体对象
        Type typeInfo;//实体对象类型
        MAction action = null;
        public OrmBase()
        {

        }
        /// <summary>
        /// 初始化状态[继承此基类的实体在构造函数中需调用此方法]
        /// </summary>
        /// <param name="entityInstance">实体对象,一般写:this</param>
        /// <param name="tableName">表名,如:Users</param>
        /// <param name="conn">数据链接,单数据库时可写Null,或写链接配置项:"Conn",或直接数据库链接字符串</param>
        protected void SetInit(Object entityInstance, string tableName, string conn)
        {
            entity = entityInstance;
            typeInfo = entity.GetType();
            action = new MAction(tableName, conn);
            action.EndTransation();
        }

        private void SetValueToAction()
        {
            PropertyInfo[] pis = typeInfo.GetProperties();
            object propValue = null;
            for (int i = 0; i < pis.Length; i++)
            {
                propValue = pis[i].GetValue(entity, null);
                if (propValue == null || propValue==DBNull.Value || Convert.ToString(propValue) == Convert.ToString(DateTime.MinValue))
                {
                    continue;
                }
                action.Data[pis[i].Name].Value = propValue;
            }
        }
        private void SetValueToEntity()
        {
            PropertyInfo[] pis = typeInfo.GetProperties();
            MDataCell cell = null;
            for (int i = 0; i < pis.Length; i++)
            {
                cell = action.Data[pis[i].Name];
                if (cell == null || cell._CellValue.IsNull)
                {
                    continue;
                }
                pis[i].SetValue(entity, cell.Value, null);
            }
        }

        #region ICommon 成员

        public bool Insert()
        {
            SetValueToAction();
            bool result = action.Insert();
            SetValueToEntity();
            return result;
        }

        public bool Update()
        {
            SetValueToAction();
            return action.Update();
        }

        public bool Update(object where)
        {
            SetValueToAction();
            return action.Update(where);
        }

        public bool Delete(object where)
        {
            return action.Delete(where);
        }

        public bool Fill(object where)
        {
            bool result = action.Fill(where);
            if (result)
            {
                SetValueToEntity();
            }
            return result;
        }
        public MDataTable Select()
        {
            int count;
            return Select(0, 0, string.Empty, out count);
        }
        public MDataTable Select(string where)
        {
            int count;
            return Select(0, 0, where, out count);
        }
        public MDataTable Select(int pageIndex, int pageSize, string where, out int count)
        {
            return action.Select(pageIndex, pageSize, where, out count);
        }


        public int GetCount(string where)
        {
            return action.GetCount(where);
        }
        public bool Exists(string where)
        {
            return action.Exists(where);
        }
        #endregion

        #region IDisposable 成员

        public void Dispose()
        {
            if (action != null)
            {
                action.Close();
            }
        }

        #endregion


    }



}
