using System;
using System.Data;

namespace CYQ.Data.SQL
{
    /// <summary>
    /// ת
    /// </summary>
    internal class DataType
    {

        internal static Type GetType(SqlDbType sqlType)
        {
            switch (sqlType)
            {
                case SqlDbType.BigInt:
                    return typeof(Int64);
                case SqlDbType.Binary:
                case SqlDbType.Image:
                case SqlDbType.VarBinary:
                    return typeof(byte[]);
                case SqlDbType.Bit:
                    return typeof(Boolean);
                case SqlDbType.Text:
                case SqlDbType.Char:
                case SqlDbType.NChar:
                case SqlDbType.NText:
                case SqlDbType.NVarChar:
                case SqlDbType.VarChar:
                    return typeof(String);
                case SqlDbType.SmallDateTime:
                case SqlDbType.DateTimeOffset:
                case SqlDbType.DateTime2:
                case SqlDbType.DateTime:
                case SqlDbType.Date:
                case SqlDbType.Timestamp:
                    return typeof(DateTime);
                case SqlDbType.Money:
                case SqlDbType.Decimal:
                case SqlDbType.SmallMoney:
                    return typeof(Decimal);
                case SqlDbType.Float:
                    return typeof(double);
                case SqlDbType.Int:
                    return typeof(int);
                case SqlDbType.Real:
                    return typeof(Single);
                case SqlDbType.TinyInt:
                    return typeof(Byte);
                case SqlDbType.SmallInt:
                    return typeof(Int16);
                case SqlDbType.UniqueIdentifier:
                    return typeof(Guid);
                default:
                    return typeof(object);
            }
        }
        /// <summary>
        /// DbTypeͶӦӳ䵽SqlDbType
        /// </summary>
        /// <param name="type">DbType</param>
        /// <returns></returns>
        public static SqlDbType GetSqlType(Type type)
        {
            return GetSqlType(type.Name.ToString());
        }
        /// <summary>
        /// DbTypeַ﷽ʽӦӳ䵽SqlDbType
        /// </summary>
        /// <param name="typeName"></param>
        /// <returns></returns>
        public static SqlDbType GetSqlType(string typeName)
        {
            switch (typeName.ToLower())
            {
                case "char":
                case "ansistringfixedlength":
                    return SqlDbType.Char;
                case "nchar":
                case "stringfixedlength":
                    return SqlDbType.NChar;
                case "varchar":
                case "ansistring":
                    return SqlDbType.VarChar;
                case "nvarchar":
                case "nvarchar2":
                case "string":
                case "clob":
                case "nclob":
                    return SqlDbType.NVarChar;
                case "timestamp":
                    return SqlDbType.Timestamp;
                case "binary":
                case "image":
                case "varbinary":
                case "blob":
                case "binary_double":
                case "binary_float":
                case "byte[]":
                    return SqlDbType.Binary;
                case "bit":
                case "boolean":
                    return SqlDbType.Bit;
                case "tinyint":
                case "byte":
                case "sbyte":
                    return SqlDbType.TinyInt;
                case "money":
                case "currency":
                    return SqlDbType.Money;
                case "datetime":
                case "date":
                    return SqlDbType.DateTime;
                case "decimal":
                case "numeric":
                    return SqlDbType.Decimal;
                case "real":
                case "double":
                    return SqlDbType.Real;
                case "uniqueidentifier":
                case "guid":
                    return SqlDbType.UniqueIdentifier;
                case "smallint":
                case "int16":
                case "uint16":
                    return SqlDbType.SmallInt;
                case "int":
                case "int32":
                case "uint32":
                case "number":
                case "integer":
                    return SqlDbType.Int;
                case "bigint":
                case "int64":
                case "uint64":
                case "varnumeric":
                    return SqlDbType.BigInt;
                case "variant":
                case "object":
                    return SqlDbType.Variant;
                case "float":
                case "single":
                    return SqlDbType.Float;
                case "smalldatetime":
                    return SqlDbType.SmallDateTime;
                case "time":
                    return SqlDbType.Time;
                case "xml":
                    return SqlDbType.Xml;
                case "ntext":
                    return SqlDbType.NText;
                case "text":
                    return SqlDbType.Text;
            }

            return SqlDbType.NVarChar;
        }


        /// <summary>
        /// DbTypeַʽӦӳ䵽DbType
        /// </summary>
        /// <param name="typeName"></param>
        /// <returns></returns>
        public static DbType GetDbType(string typeName)
        {
            switch (typeName.ToLower())
            {
                case "ansistring":
                    return DbType.AnsiString;
                case "ansistringfixedlength":
                    return DbType.AnsiStringFixedLength;
                case "varbinary":
                case "binary":
                case "image":
                case "timestamp":
                case "byte[]":
                    return DbType.Binary;
                case "bit":
                case "boolean":
                    return DbType.Boolean;
                case "tinyint":
                case "byte":
                    return DbType.Byte;
                case "smallmoney":
                case "currency":
                    return DbType.Currency;

                case "date":
                    return DbType.Date;
                case "smalldatetime":
                case "datetime":
                    return DbType.DateTime;
                case "numeric":
                case "decimal":
                    return DbType.Decimal;
                case "money":
                case "double":
                    return DbType.Double;
                case "uniqueidentifier":
                case "guid":
                    return DbType.Guid;
                case "smallint":
                case "int16":
                case "uint16":
                    return DbType.Int16;
                case "int":
                case "int32":
                case "uint32":
                    return DbType.Int32;
                case "bigint":
                case "int64":
                case "uint64":
                    return DbType.Int64;
                case "variant":
                case "object":
                    return DbType.Object;
                case "sbyte":
                    return DbType.SByte;
                case "float":
                case "single":
                    return DbType.Single;
                case "char":
                    return DbType.AnsiStringFixedLength;
                case "nchar":
                case "stringfixedlength":
                    return DbType.StringFixedLength;
                case "varchar":
                    return DbType.AnsiString;
                case "text":
                case "ntext":
                case "string":
                case "nvarchar":
                    return DbType.String;
                case "time":
                    return DbType.Time;
                case "varnumeric":
                    return DbType.VarNumeric;
                case "xml":
                    return DbType.Xml;
            }

            return DbType.Object;
        }
        /// <summary>
        /// SqlDbTypeͶӦӳ䵽DbType
        /// </summary>
        /// <param name="type">SqlDbType</param>
        /// <returns></returns>
        public static DbType GetDbType(Type type)
        {
            return GetDbType(type.Name.ToString());
        }

        #region 

        /// <summary>
        /// ĸͷ0ͷ1ͷ2bool3999
        /// </summary>
        /// <param name="sqlDbType"></param>
        /// <returns></returns>
        internal static int GetGroupID(SqlDbType sqlDbType)
        {
            switch (sqlDbType)
            {
                case SqlDbType.Int:
                case SqlDbType.TinyInt:
                case SqlDbType.BigInt:
                case SqlDbType.SmallInt:
                case SqlDbType.Float:
                case SqlDbType.Real:
                case SqlDbType.Decimal:
                case SqlDbType.Money:
                    return 1;
                case SqlDbType.UniqueIdentifier:
                case SqlDbType.NVarChar:
                case SqlDbType.VarChar:
                case SqlDbType.NChar:
                case SqlDbType.Char:
                case SqlDbType.Text:
                case SqlDbType.NText:
                    return 0;
                case SqlDbType.DateTime:
                case SqlDbType.SmallDateTime:
                case SqlDbType.Timestamp:
                    return 2;
                case SqlDbType.Bit:
                    return 3;
                default:
                    return 999;
            }

        }
        #endregion
    }
}
