﻿using System;
using CYQ.Data.Xml;
using System.Xml;
using System.Web;
namespace CYQ.Data.Xml
{
    public class MutilLanguage : IDisposable
    {

        XHtmlAction helper;
        //public string DefaultValue = "no find";
        private LanguageKey _LanKey = LanguageKey.None;
        public LanguageKey LanKey
        {
            get
            {
                if (_LanKey == LanguageKey.None)
                {
                    SetDefaultLangKey();
                }
                return _LanKey;
            }
            set
            {
                _LanKey = value;
            }
        }
        public MutilLanguage(string filePath, bool forHtml, bool getValueByCookie)
        {
            Init(filePath, forHtml, getValueByCookie);
        }
        public MutilLanguage(string filePath, bool forHtml)
        {
            Init(filePath, forHtml, true);
        }
        public MutilLanguage(string filePath)
        {
            Init(filePath, true, true);
        }
        private void Init(string filePath, bool forHtml, bool getValueByCookie)
        {
            helper = new XHtmlAction(forHtml);
            if (!helper.Load(filePath))
            {
                throw new Exception("Load xml failed : " + filePath);
            }
            if (getValueByCookie)
            {
                GetFromCookie();
            }
        }
        public string Get(object lanID)
        {
            return Get(lanID, LanKey);
        }
        public string Get(object lanID, int lanKeyID)
        {
            if (lanKeyID > 0 && lanKeyID < 10)
            {
                return Get(lanID, (LanguageKey)lanKeyID);
            }
            return Get(lanID);
        }
        public string Get(object lanID, LanguageKey lanEnum)
        {
            XmlNode node = helper.GetByID(Convert.ToString(lanID));
            if (node != null)
            {
                switch (lanEnum)
                {
                    case LanguageKey.China:
                        return node.InnerXml.Trim('\r', '\n').Trim();
                    default:
                        string key = lanEnum.ToString().ToLower().Substring(0, 3);
                        if (node.Attributes[key] != null)
                        {
                            return node.Attributes[key].Value.Trim('\r', '\n').Trim();
                        }
                        else
                        {
                            return node.InnerXml.Trim('\r', '\n').Trim();
                        }
                }
            }
            return Convert.ToString(lanID);
        }
        private void GetFromCookie()
        {
            HttpCookie myCookie = HttpContext.Current.Request.Cookies[AppConfig.Domain + "_LanKey"];
            if (null != myCookie)
            {
                try
                {
                    _LanKey = (LanguageKey)Enum.Parse(typeof(LanguageKey), myCookie.Value);
                }
                catch
                {
                    _LanKey = LanguageKey.None;
                }
            }
        }
        public void SetToCookie(LanguageKey lanKey)
        {
            SetToCookie(lanKey.ToString());
        }
        public void SetToCookie(string lanKey)
        {
            try
            {
                _LanKey = (LanguageKey)Enum.Parse(typeof(LanguageKey), lanKey);
            }
            catch
            {
                _LanKey = LanguageKey.None;
            }
            HttpCookie myCookie = new HttpCookie(AppConfig.Domain + "_LanKey", lanKey);
            if (!string.IsNullOrEmpty(AppConfig.Domain) && AppConfig.Domain.IndexOf(':') == -1)//端口处理
            {
                myCookie.Domain = AppConfig.Domain;
            }
            myCookie.Expires = System.DateTime.Now.AddHours(1);
            HttpContext.Current.Response.Cookies.Add(myCookie);
        }
        public static string GetKey(int value)
        {
            if (value > 0 && value < 10)
            {
                return Convert.ToString((LanguageKey)value);
            }
            return LanguageKey.None.ToString();
        }
        public static int GetValue(string langKey)
        {
            switch (langKey)
            {
                case "China":
                    return 1;
                case "English":
                    return 2;
                case "French":
                    return 3;
                case "German":
                    return 4;
                case "Hindi":
                    return 5;
                case "Italian":
                    return 6;
                case "Japanese":
                    return 7;
                case "Korean":
                    return 8;
                case "Russian":
                    return 9;
                case "Custom":
                    return 10;

            }
            return 0;
        }
        private void SetDefaultLangKey()
        {
            if (HttpContext.Current.Request.UserLanguages != null && HttpContext.Current.Request.UserLanguages.Length > 0)
            {
                switch (HttpContext.Current.Request.UserLanguages[0].Substring(0, 2))
                {
                    case "zh":
                        _LanKey = LanguageKey.China;
                        break;
                    case "en":
                        _LanKey = LanguageKey.English;
                        break;
                    case "fr":
                        _LanKey = LanguageKey.French;
                        break;
                    case "de":
                        _LanKey = LanguageKey.German;
                        break;
                    case "hi":
                        _LanKey = LanguageKey.Hindi;
                        break;
                    case "it":
                        _LanKey = LanguageKey.Italian;
                        break;
                    case "ja":
                        _LanKey = LanguageKey.Japanese;
                        break;
                    case "ko":
                        _LanKey = LanguageKey.Korean;
                        break;
                    case "ru":
                        _LanKey = LanguageKey.Russian;
                        break;
                }
            }
            if (_LanKey == LanguageKey.None)
            {
                string key = AppConfig.SysLangKey;
                if (string.IsNullOrEmpty(key))
                {
                    _LanKey = LanguageKey.China;
                }
                else
                {
                    try
                    {
                        _LanKey = (LanguageKey)Enum.Parse(typeof(LanguageKey), key);
                    }
                    catch
                    {
                        throw new Exception(string.Format("Error:LanguageKey not contain {0}", key));
                    }
                }
            }
        }
        #region IDisposable 成员

        public void Dispose()
        {
            helper.Dispose();
        }

        #endregion
    }
    public enum LanguageKey
    {
        /// <summary>
        /// 未设置状态
        /// </summary>
        None = 0,
        /// <summary>
        /// 中文
        /// </summary>
        China = 1,
        /// <summary>
        /// 英文
        /// </summary>
        English = 2,
        /// <summary>
        /// 法语
        /// </summary>
        French = 3,

        /// <summary>
        /// 德语
        /// </summary>
        German = 4,

        /// <summary>
        /// 韩语
        /// </summary>
        Korean = 5,

        /// <summary>
        /// 日语
        /// </summary>
        Japanese = 6,

        /// <summary>
        /// 印地语
        /// </summary>
        Hindi = 7,

        /// <summary>
        ///  俄语
        /// </summary>
        Russian = 8,

        /// <summary>
        /// 意大利语
        /// </summary>
        Italian = 9,
        /// <summary>
        /// 自定义语言
        /// </summary>
        Custom = 10
    }
}
