using System;
using System.Xml;
using CYQ.Data.Table;

namespace CYQ.Data.Xml
{
    public class XHtmlAction : XHtmlBase
    {
        public XHtmlAction()
            : base()
        {

        }
        public XHtmlAction(bool forHtml)
            : base()
        {
            if (forHtml)
            {
                base.LoadNameSpace(htmlNameSpace);
            }
        }
        public XHtmlAction(string nameSpaceUrl)
            : base()
        {
            base.LoadNameSpace(nameSpaceUrl);
        }
        #region 
        MDataRow _Row;
        MDataTable _Table;



        #region رѭʽ
        public void LoadData(MDataTable table)
        {
            _Table = table;
            if (_Table.Rows.Count > 0)
            {
                _Row = _Table.Rows[0];
            }
        }
        public delegate string SetForeachEventHandler(string text, object[] values, int row);
        public event SetForeachEventHandler OnForeach;
        public void SetForeach(string id, SetType setType, params object[] formatValues)
        {
            string text = string.Empty;
            XmlNode node = GetByID(id);
            if (node == null)
            {
                return;
            }
            switch (setType)
            {
                case SetType.InnerText:
                    text = node.InnerText;
                    break;
                case SetType.InnerXml:
                    text = node.InnerXml;
                    break;
                case SetType.Value:
                case SetType.Href:
                case SetType.Src:
                case SetType.Class:
                    string key = setType.ToString().ToLower();
                    if (node.Attributes[key] != null)
                    {
                        text = node.Attributes[key].Value;
                    }
                    break;
            }
            SetForeach(node, text, formatValues);
        }
        public void SetForeach(string id, string text, params object[] formatValues)
        {
            XmlNode node = GetByID(id);
            SetForeach(node, text, formatValues);
        }
        private void SetForeach(XmlNode node, string text, params object[] formatValues)
        {
            if (node != null && _Table != null && _Table.Rows.Count > 0)
            {
                string innerXml = "";
                object[] values = new object[formatValues.Length];
                //foreach (MDataRow row in _Table.Rows)
                string newText = text;
                MDataCell cell;
                for (int k = 0; k < _Table.Rows.Count; k++)
                {
                    for (int i = 0; i < values.Length; i++)
                    {
                        if (formatValues[i] == null)
                        {
                            continue;
                        }
                        cell = _Table.Rows[k][formatValues[i].ToString()];
                        if (cell == null && formatValues[i].ToString().ToLower() == "row")
                        {
                            values[i] = k + 1;
                        }
                        else if (cell != null)
                        {
                            values[i] = cell.Value;
                        }
                    }
                    if (OnForeach != null)
                    {
                        newText = OnForeach(text, values, k);
                    }
                    try
                    {
                        innerXml += string.Format(newText, values);
                    }
                    catch
                    {

                    }
                }
                try
                {
                    node.InnerXml = innerXml;
                }
                catch
                {
                    node.InnerXml = SetCDATA(innerXml);
                }
            }
            if (OnForeach != null)
            {
                OnForeach = null;
            }
        }
        #endregion

        #region ݺʽ

        private bool _IsUserLang = true;
        /// <summary>
        /// ǰǷû
        /// </summary>
        public bool IsUserLang
        {
            get
            {
                return _IsUserLang;
            }
            set
            {
                _IsUserLang = value;
            }
        }
        public bool IsOpenUserLang = true;//ǷʼԶԷָ
        public void LoadData(MDataRow row)
        {
            _Row = row;
        }
        public void SetFor(string id)
        {
            SetFor(id, SetType.InnerXml);
        }
        public void SetFor(string id, SetType setType)
        {
            SetFor(id, setType, GetRowValue(id));
        }
        public void SetFor(string id, SetType setType, params string[] values)
        {
            int i = setType == SetType.Custom ? 1 : 0;
            for (; i < values.Length; i++)
            {
                if (values[i].Contains(ValueReplace.New))
                {
                    values[i] = values[i].Replace(ValueReplace.New, GetRowValue(id));
                }
            }
            XmlNode node = GetByID(id);

            Set(node, setType, values);
        }
        private string GetRowValue(string id)
        {
            string rowValue = "";
            if (_Row != null)
            {
                MDataCell cell = _Row[id.Substring(3)];
                if (cell == null)
                {
                    cell = _Row[id];
                }
                if (cell != null)
                {
                    rowValue = Convert.ToString(cell.Value);
                }
            }
            return rowValue;
        }
        #endregion
        private string SetValue(string sourceValue, string newValue, bool addCData)
        {
            if (string.IsNullOrEmpty(newValue))
            {
                return sourceValue;
            }
            newValue = newValue.Replace(ValueReplace.Source, sourceValue);
            if (IsOpenUserLang)
            {
                int split = newValue.IndexOf(ValueReplace.LangSplit);
                if (split > -1)
                {
                    newValue = _IsUserLang ? newValue.Substring(0, split) : newValue.Substring(split + ValueReplace.LangSplit.Length);
                }
            }
            if (addCData)
            {
                newValue = SetCDATA(newValue);
            }
            return newValue;
        }
        private void SetAttrValue(XmlNode node, string key, string value)
        {
            if (node.Attributes[key] == null)
            {
                XmlAttribute attr = xmlDoc.CreateAttribute(key);
                node.Attributes.Append(attr);
            }
            node.Attributes[key].Value = SetValue(node.Attributes[key].InnerXml, value, false);
        }

        public void Set(XmlNode node, SetType setType, params string[] values)
        {

            if (node != null)
            {
                switch (setType)
                {
                    case SetType.InnerText:
                        node.InnerText = SetValue(node.InnerText, values[0], false);
                        break;
                    case SetType.InnerXml:
                        node.InnerXml = SetValue(node.InnerXml, values[0], true);
                        break;
                    case SetType.Value:
                    case SetType.Href:
                    case SetType.Src:
                    case SetType.Class:
                    case SetType.Disabled:
                    case SetType.ID:
                    case SetType.Name:
                        string key = setType.ToString().ToLower();
                        SetAttrValue(node, key, values[0]);
                        break;
                    case SetType.Custom:
                        SetAttrValue(node, values[0], values[1]);
                        break;
                    case SetType.A:
                        node.InnerXml = SetValue(node.InnerXml, values[0], true);
                        if (values.Length > 1)
                        {
                            SetAttrValue(node, "href", values[1]);
                        }
                        break;
                    case SetType.Select:
                        foreach (XmlNode option in node.ChildNodes)
                        {
                            if (option.Attributes["value"] != null && (option.Attributes["value"].Value == values[0] || option.Attributes["value"].Value.Split(',')[0] == values[0]))
                            {
                                SetAttrValue(option, "selected", "selected");
                                break;
                            }
                        }
                        break;
                    case SetType.Checked:
                        if (values[0] == "1" || values[0].ToLower() == "true")
                        {
                            key = setType.ToString().ToLower();
                            SetAttrValue(node, key, key);
                        }
                        break;
                }
            }
        }
        public void Set(string id, SetType setType, params string[] values)
        {
            XmlNode node = GetByID(id);
            Set(node, setType, values);
        }
        public void Set(string id, string value)
        {
            XmlNode node = GetByID(id);
            Set(node, SetType.InnerXml, value);
        }
        #endregion
        #region ѯ
        public bool Contains(string id)
        {
            return GetByID(id) != null;
        }
        public XmlNode GetByName(string name)
        {
            return Fill(GetXPath("*", "name", name), null);
        }
        public XmlNode GetByName(string name, XmlNode node)
        {
            return Fill(GetXPath("*", "name", name), node);
        }
        public XmlNode GetByID(string id)
        {
            return Fill(GetXPath("*", "id", id), null);
        }
        public XmlNode GetByID(string id, XmlNode node)
        {
            return Fill(GetXPath("*", "id", id), node);
        }
        public XmlNode Get(string tag, string attr, string value, XmlNode parent)
        {
            return Fill(GetXPath(tag, attr, value), parent);
        }



        public XmlNodeList GetList(string tag, string attr, string value)
        {
            return Select(GetXPath(tag, attr, value), null);
        }
        public XmlNodeList GetList(string tag, string attr, string value, XmlNode node)
        {
            return Select(GetXPath(tag, attr, value), node);
        }
        public XmlNodeList GetList(string tag, string attr)
        {
            return Select(GetXPath(tag, attr, null), null);
        }
        public XmlNodeList GetList(string tag, string attr, XmlNode node)
        {
            return Select(GetXPath(tag, attr, null), node);
        }
        public XmlNodeList GetList(string tag)
        {
            return Select(GetXPath(tag, null, null), null);
        }
        public XmlNodeList GetList(string tag, XmlNode node)
        {
            return Select(GetXPath(tag, null, null), node);
        }
        #endregion

        #region 
        public void CreateNodeTo(XmlNode parent,string tag, string text, params string[] attrAndValue)
        {
            if (parent != null)
            {
                parent.AppendChild(CreateNode(tag, text, attrAndValue));
            }
        }
        public XmlNode CreateNode(string tag, string text, params string[] attrAndValue)
        {
            XmlElement xElement = Create(tag);
            try
            {
                xElement.InnerXml = text;
            }
            catch
            {
                xElement.InnerXml = SetCDATA(text);
            }
            if (attrAndValue != null && attrAndValue.Length % 2 == 0)
            {
                string attr = "", value = "";
                for (int i = 0; i < attrAndValue.Length; i++)
                {
                    attr = attrAndValue[i];
                    i++;
                    value = attrAndValue[i];
                    xElement.SetAttribute(attr, value);
                }
            }
            return xElement as XmlNode;
        }
        #endregion

        #region 
        public void AppendNode(XmlNode parentNode, XmlNode childNode)
        {
            if (parentNode != null && childNode != null)
            {
                parentNode.AppendChild(childNode);
            }
        }
        public void AppendNode(XmlNode parentNode, XmlNode childNode, int position)
        {
            if (parentNode != null && childNode != null)// A B
            {
                if (parentNode.ChildNodes.Count == 0 || position >= parentNode.ChildNodes.Count)
                {
                    parentNode.AppendChild(childNode);
                }
                else if (position == 0)
                {
                    InsertBefore(childNode, parentNode.ChildNodes[0]);
                }
                else
                {
                    InsertAfter(childNode, parentNode.ChildNodes[position - 1]);
                }
            }
        }

        #endregion

        #region ɾ
        /// <summary>
        /// ڵ,ڵ/
        /// </summary>
        /// <param name="OldNode"></param>
        public void Clear(XmlNode node)
        {
            node.RemoveAll();
        }
        public void Remove(XmlNode node)
        {
            node.ParentNode.RemoveChild(node);
        }
        public void Remove(string id)
        {
            XmlNode node = GetByID(id);
            if (node != null)
            {
                node.ParentNode.RemoveChild(node);
            }
        }
        public void RemoveChild(string id, int index)
        {
            XmlNode node = GetByID(id);
            if (node != null)
            {
                RemoveChild(node, index);
            }
        }
        public void RemoveChild(XmlNode node, int index)
        {
            if (node.ChildNodes.Count >= index)
            {
                node.RemoveChild(node.ChildNodes[index - 1]);
            }
        }
        #endregion

        #region ڵ/ڵ

        /// <summary>
        /// ڵ㽻λ
        /// </summary>
        /// <param name="XNodeFirst">һڵ</param>
        /// <param name="XNodeLast">ڶڵ</param>
        public void InterChange(XmlNode xNodeFirst, XmlNode xNodeLast)
        {
            if (xNodeFirst != null && xNodeLast != null)
            {
                if (xNodeFirst.ParentNode != null && xNodeLast.ParentNode != null)
                {
                    xNodeFirst.ParentNode.ReplaceChild(xNodeLast.Clone(), xNodeFirst);
                    xNodeLast.ParentNode.ReplaceChild(xNodeFirst.Clone(), xNodeLast);
                }
                else
                {
                    xmlDoc.DocumentElement.ReplaceChild(xNodeLast.Clone(), xNodeFirst);
                    xmlDoc.DocumentElement.ReplaceChild(xNodeFirst.Clone(), xNodeLast);
                }
            }
        }
        /// <summary>
        /// ڵ滻[֧ĵ滻]
        /// </summary>
        /// <param name="NewXNode"></param>
        /// <param name="OldXNode"></param>
        public void ReplaceNode(XmlNode newNode, XmlNode oldNode)
        {
            if (newNode != null && oldNode != null)
            {
                oldNode.RemoveAll();
                //if (!string.IsNullOrEmpty(newNode.InnerXml) && !string.IsNullOrEmpty(newNode.NamespaceURI))
                //{
                //    oldNode.InnerXml = newNode.InnerXml.Replace("xmlns=\"" + newNode.NamespaceURI+"\"", string.Empty);
                //}
                //else
                //{
                oldNode.InnerXml = newNode.InnerXml;
                //}
                XmlAttributeCollection xAttributes = newNode.Attributes;
                if (xAttributes != null && xAttributes.Count > 0)
                {
                    for (int i = 0; i < xAttributes.Count; i++)
                    {
                        ((XmlElement)oldNode).SetAttribute(xAttributes[i].Name, xAttributes[i].Value);
                    }
                }
            }
        }
        /// <summary>
        /// ڵ֮[֧ĵ֮Ĳ]
        /// </summary>
        /// <param name="NewNode">Ҫ½ڵ</param>
        /// <param name="RefNode">ڴ˽ڵNewNodeڵ</param>
        public void InsertAfter(XmlNode newNode, XmlNode refNode)
        {
            XmlNode xDocNode = CreateNode(newNode.Name, "");
            ReplaceNode(newNode, xDocNode);
            refNode.ParentNode.InsertAfter(xDocNode, refNode);
        }
        /// <summary>
        /// ڵ֮ǰ[֧ĵ֮Ĳ]
        /// </summary>
        /// <param name="NewNode">Ҫ½ڵ</param>
        /// <param name="RefNode">ڴ˽ڵǰNewNodeڵ</param>
        public void InsertBefore(XmlNode newNode, XmlNode refNode)
        {
            XmlNode xDocNode = CreateNode(newNode.Name, "");
            ReplaceNode(newNode, xDocNode);
            refNode.ParentNode.InsertBefore(xDocNode, refNode);
        }
        #endregion

        #region ж/ȡֵ
        public bool HasAttr(string nodeID, string attrName)
        {
            return GetAttrValue(nodeID, attrName) != string.Empty;
        }
        public bool HasAttr(XmlNode node, string attrName)
        {
            return GetAttrValue(node, attrName) != string.Empty;
        }
        public string GetAttrValue(string nodeID, string attrName)
        {
            XmlNode node = GetByID(nodeID);
            return GetAttrValue(node, attrName);
        }
        public string GetAttrValue(XmlNode node, string attrName)
        {
            if (node != null && node.Attributes != null && node.Attributes[attrName] != null)
            {
                return node.Attributes[attrName].Value;
            }
            return string.Empty;
        }
        public void RemoveAttr(string nodeID, params string[] attrNames)
        {
            XmlNode node = GetByID(nodeID);
            RemoveAttr(node, attrNames);
        }
        public void RemoveAttr(XmlNode node, params string[] attrNames)
        {
            if (node != null && node.Attributes != null)
            {
                foreach (string name in attrNames)
                {
                    if (node.Attributes[name] != null)
                    {
                        node.Attributes.Remove(node.Attributes[name]);
                    }
                }

            }
        }

        #endregion
    }

}
